--- loncom/interface/lonhelper.pm	2006/07/17 16:26:09	1.157
+++ loncom/interface/lonhelper.pm	2007/07/25 23:20:38	1.161
@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # .helper XML handler to implement the LON-CAPA helper
 #
-# $Id: lonhelper.pm,v 1.157 2006/07/17 16:26:09 raeburn Exp $
+# $Id: lonhelper.pm,v 1.161 2007/07/25 23:20:38 albertel Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -81,25 +81,28 @@ State tags are also required to have an
 human name of the state, and will be displayed as the header on top of 
 the screen for the user.
 
+State tags may also optionally have an attribute "help" which should be
+the filename of a help file, this will add a blue ? to the title.
+
 =head2 Example Helper Skeleton
 
 An example of the tags so far:
 
  <helper title="Example Helper">
    <state name="START" title="Demonstrating the Example Helper">
-     <!-- notice this is the START state the wizard requires -->
+     <!-- notice this is the START state the helper requires -->
      </state>
    <state name="GET_NAME" title="Enter Student Name">
      </state>
    </helper>
 
-Of course this does nothing. In order for the wizard to do something, it is
-necessary to put actual elements into the wizard. Documentation for each
+Of course this does nothing. In order for the helper to do something, it is
+necessary to put actual elements into the helper. Documentation for each
 of these elements follows.
 
 =head1 Creating a Helper With Code, Not XML
 
-In some situations, such as the printing wizard (see lonprintout.pm), 
+In some situations, such as the printing helper (see lonprintout.pm), 
 writing the helper in XML would be too complicated, because of scope 
 issues or the fact that the code actually outweighs the XML. It is
 possible to create a helper via code, though it is a little odd.
@@ -334,7 +337,8 @@ sub start_state {
     }
 
     Apache::lonhelper::state->new($token->[2]{'name'},
-                                  $token->[2]{'title'});
+                                  $token->[2]{'title'},
+				  $token->[2]{'help'});
     return '';
 }
 
@@ -577,6 +581,7 @@ sub display {
 
     # Phase 4: Display.
     my $stateTitle=&mt($state->title());
+    my $stateHelp=     $state->help();
     my $browser_searcher_js = 
 	'<script type="text/javascript">'."\n".
 	&Apache::loncommon::browser_and_searcher_javascript().
@@ -591,9 +596,12 @@ sub display {
 
 
     if (!$state->overrideForm()) { $result.="<form name='helpform' method='POST'>"; }
+    if ($stateHelp) {
+	$stateHelp = &Apache::loncommon::help_open_topic($stateHelp);
+    }
     $result .= <<HEADER;
         <table border="0" width='100%'><tr><td>
-        <h2><i>$stateTitle</i></h2>
+        <h2><i>$stateTitle</i>$stateHelp</h2>
 HEADER
 
     $result .= "<table cellpadding='10' width='100%'><tr><td rowspan='2' valign='top'>";
@@ -680,6 +688,7 @@ sub new {
 
     $self->{NAME} = shift;
     $self->{TITLE} = shift;
+    $self->{HELP} = shift;
     $self->{ELEMENTS} = [];
 
     bless($self, $class);
@@ -701,6 +710,11 @@ sub title {
     return $self->{TITLE};
 }
 
+sub help {
+    my $self = shift;
+    return $self->{HELP};
+}
+
 sub preprocess {
     my $self = shift;
     for my $element (@{$self->{ELEMENTS}}) {
@@ -792,7 +806,7 @@ the element. How this value is interpret
 the element itself, and possibly the settings the element has (such as 
 multichoice vs. single choice for <choices> tags). 
 
-This is also intended for things like the course initialization wizard, where the
+This is also intended for things like the course initialization helper, where the
 user is setting various parameters. By correctly grabbing current settings 
 and including them into the helper, it allows the user to come back to the
 helper later and re-execute it, without needing to worry about overwriting
@@ -1010,7 +1024,7 @@ sub end_message {
 sub render {
     my $self = shift;
 
-    return &mtn($self->{MESSAGE_TEXT});
+    return $self->{MESSAGE_TEXT};
 }
 # If a NEXTSTATE was given, switch to it
 sub postprocess {
@@ -1023,6 +1037,100 @@ sub postprocess {
 }
 1;
 
+package Apache::lonhelper::helpicon;
+
+=pod
+
+=head1 Elements
+
+=head2 Element: helpiconX<helpicon, helper element>
+
+Helpicon elements add a help icon at the current location.
+Example:
+
+   <helpicon file="Help">
+     General Help
+   </helpicon>
+
+In this example will generate a help icon to the Help.hlp url with a
+description of 'General Help'. The description is not required and if
+left out (Example: <helpicon file="Help" /> only the icon will be
+added.)
+
+=head3 Localization
+
+The description text will be run through the normalize_string function
+and that will be used as a call to &mt.
+
+=cut
+
+no strict;
+@ISA = ("Apache::lonhelper::element");
+use strict;
+use Apache::lonlocal;
+
+BEGIN {
+    &Apache::lonhelper::register('Apache::lonhelper::helpicon',
+				 ('helpicon'));
+}
+
+sub new {
+    my $ref = Apache::lonhelper::element->new();
+    bless($ref);
+}
+
+# CONSTRUCTION: Construct the message element from the XML
+sub start_helpicon {
+    my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
+
+    if ($target ne 'helper') {
+        return '';
+    }
+
+    $paramHash->{HELP_TEXT} = &mtn(&Apache::lonxml::get_all_text('/helpicon',
+								 $parser));
+
+    $paramHash->{HELP_TEXT} =~s/^\s+//;
+    $paramHash->{HELP_TEXT} =~s/\s+$//;
+
+    if (defined($token->[2]{'file'})) {
+        $paramHash->{HELP_FILE} = $token->[2]{'file'};
+    }
+    return '';
+}
+
+sub end_helpicon {
+    my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
+
+    if ($target ne 'helper') {
+        return '';
+    }
+    Apache::lonhelper::helpicon->new();
+    return '';
+}
+
+sub render {
+    my $self = shift;
+
+    my $text;
+    if ( $self->{HELP_TEXT} ne '') {
+	$text=&mtn($self->{HELP_TEXT});
+    }
+
+    return &Apache::loncommon::help_open_topic($self->{HELP_FILE},
+					       $text);
+}
+sub postprocess {
+    my $self = shift;
+    if (defined($self->{NEXTSTATE})) {
+        $helper->changeState($self->{NEXTSTATE});
+    }
+
+    return 1;
+}
+
+1;
+
 package Apache::lonhelper::skip;
 
 =pod
@@ -1369,7 +1477,7 @@ BUTTONS
             $choiceLabel = &$choiceLabel($helper, $self);
         }
         $result .= "/></td><td> ".qq{<label for="id$id">}.
-            $choiceLabel. "</label></td>";
+            &mtn($choiceLabel). "</label></td>";
 	if ($choice->[4]) {
 	    $result .='<td><input type="text" size="5" name="'
 		.$choice->[4].'_forminput" value="'
@@ -2430,7 +2538,7 @@ sub render {
 
     #   Current personel
 
-    $result .= '<h4>Select Currently Enrolled Students and Active Course Personnel</h4>';
+    $result .= '<h4>'.&mt('Select Currently Enrolled Students and Active Course Personnel').'</h4>';
     $result .= &Apache::lonselstudent::render_student_list( $current_members,
 							    "helpform",
 							    "current", 
@@ -2442,11 +2550,11 @@ sub render {
 
     # If activeonly is not set then we can also give the expired students:
     #
-    if (!$self->{'activeonly'} && ((scalar @$expired_members) > 0)) {
+    if (!$self->{'activeonly'} && ((scalar(@$future_members)) > 0)) {
 
 	# And future.
 
-	$result .= '<h4>Select Future Enrolled Students and Future Course Personnel</h4>';
+	$result .= '<h4>'.&mt('Select Future Enrolled Students and Future Course Personnel').'</h4>';
        
 	$result .= &Apache::lonselstudent::render_student_list( $future_members,
 								"helpform",
@@ -2455,9 +2563,11 @@ sub render {
 								$self->{'multichoice'},
 								$self->{'variable'},
 								0);
+    }
+    if (!$self->{'activeonly'} && ((scalar(@$expired_members)) > 0)) {
 	# Past 
 
-	$result .= '<h4>Select Previously Enrolled Students and Inactive Course Personnel</h4>';
+	$result .= '<h4>'.&mt('Select Previously Enrolled Students and Inactive Course Personnel').'</h4>';
 	$result .= &Apache::lonselstudent::render_student_list($expired_members,
 							       "helpform",
 							       "past",
@@ -3413,7 +3523,7 @@ sub overrideForm {
 
 package Apache::lonhelper::parmwizfinal;
 
 # This is the final state for the parmwizard. It is not generally useful,
 # so it is not perldoc'ed. It does its own processing.
 # It is represented with <parmwizfinal />, and
 # should later be moved to lonparmset.pm .
@@ -3413,7 +3523,7 @@ sub overrideForm {
 package Apache::lonhelper::parmwizfinal;
 
-# This is the final state for the parmwizard. It is not generally useful,
+# This is the final state for the parm helper. It is not generally useful,
 # so it is not perldoc'ed. It does its own processing.
 # It is represented with <parmwizfinal />, and